/******************************************************************************************************
Source data: This program downloads return and volatility data from the CRSP database.
Purpose: To calculate lagged returns for the prior qtr, prior two qtrs, prior year and prior two years.
Note: This script can take a very long time to run (38 million observations)
******************************************************************************************************/

%let wrds=wrds.wharton.upenn.edu 4016;
options comamid=TCP remote=WRDS;
signon username=_prompt_;
rsubmit;

libname crsp '/wrds/crsp/sasdata/a_stock';

/* This section creates the master file of daily equity returns */
data remote_dsf;
    set crsp.dsf;
	year=year(date);
	cnum = substr(cusip,1,6);
	if cnum = '.' then delete;
	if ret = '.' then delete;
	if ret = '.C' then delete;
	if ret = '.B' then delete;
	if year < 1992 then delete;
    keep cnum date ret year;
run;
proc download data = remote_dsf out = general.dsf;
run;

/*****************************************************************************/
/* This section calculates s.d. of daily equity returns */
proc sort data = remote_dsf;
    by cnum date;
data sd_calc;
	set remote_dsf;
	ln_ret = log(1+ret);
run;
proc sort data=sd_calc;
	by cnum year;
run;
proc means noprint data=sd_calc;
	by cnum year;
	var ln_ret;
	output out=remote_daily_sd STDDEV=/autoname;
run;
proc download data = remote_daily_sd out = general.daily_sd;
run;

/*****************************************************************************/
/* This section creates a dataset of daily index returns */
data remote_sp;
	set crsp.dsi;
	keep date vwretd sprtrn;
run;
proc sort data=remote_dsf;
	by date	cnum;
run;
proc sort data=remote_sp;
	by date;
run;
data dsf_merged;
	merge remote_dsf remote_sp;
	by date;
run; 
proc download data = remote_sp out = general.sp_returns;
run;
endrsubmit;
signoff;
