/*****************************************************************************
This program downloads data from several WRDS databases
The purpose is to collect compensation variables to calculate vega and delta.
Input scripts: "Daily stock return data";
*****************************************************************************/

%let wrds=wrds.wharton.upenn.edu 4016;
options comamid=TCP remote=WRDS;
signon username=_prompt_;
rsubmit;

libname excomp '/wrds/comp/sasdata/execcomp';
libname compx '/wrds/comp/sasdata/nam';
/*****************************************************************************/
/*
data remote_bs_data;
    set compx.funda;
	where fyear between 1990 and 2012;
	d_yld = dvpsx_f / prcc_f;
    keep gvkey cusip fyear dvpsx_f prcc_f d_yld;
run;
proc sort data = remote_bs_data;
    by gvkey fyear;
run;
data remote_bs_clean;
	set remote_bs_data;
	cnum = substr(cusip,1,6);
	if cnum = '.' then delete;
	drop cusip;
run;
proc download data = remote_bs_clean out = pilot.bs;
run;
*/

/*****************************************************************************/
data remote_anncomp_data;
    set excomp.anncomp;
	where year between 1990 and 2012;
    keep gvkey cusip year co_per_rol execid 
	CHG_CTRL_PYMT
	EIP_UNEARN_NUM
	EIP_UNEARN_VAL
	OPTION_AWARDS_BLK_VALUE
	OPTION_AWARDS_FV
	OPTION_AWARDS_NUM
	OPTION_AWARDS_RPT_VALUE
	OPT_EXER_NUM
	OPT_EXER_VAL
	OPT_UNEX_EXER_EST_VAL
	OPT_UNEX_EXER_NUM
	OPT_UNEX_UNEXER_EST_VAL
	OPT_UNEX_UNEXER_NUM
	RSTKGRNT
	SALARY
	SHROWN_EXCL_OPTS
	SHROWN_TOT
	STOCK_AWARDS
	STOCK_AWARDS_FV
	STOCK_UNVEST_NUM
	STOCK_UNVEST_VAL
	TOTAL_ALT1
	TOTAL_ALT2
	TOTAL_CURR
	TOTAL_SEC
	TDC1 
	TDC2;
run;
proc sort data = remote_anncomp_data;
    by gvkey year;
data remote_anncomp_clean;
	set remote_anncomp_data;
	cnum = substr(cusip,1,6);
	if gvkey = '.' then delete;
	if year = '.' then delete;
	cash_comp = TOTAL_CURR;
	total_comp = TDC1;
	perc_cash_comp = cash_comp/total_comp;
	drop cusip TOTAL_CURR TDC1;
run;
data remote_prcc_data;
    set compx.funda;
	where fyear between 1990 and 2012;
	year = fyear;
    keep gvkey year prcc_f;
run;
proc sort data = remote_prcc_data;
    by gvkey year;
data remote_anncomp_clean_m;
	merge remote_anncomp_clean (in=x) remote_prcc_data (in=y);
	by gvkey year;
	if x=1;
run;
proc download data = remote_anncomp_clean_m out = pilot.anncomp;
run;
/*****************************************************************************/
data remote_grant_data_pre2006;
	set excomp.stgrttab;
	cnum = substr(cusip,1,6);
	keep GVKEY
		cnum
		CO_PER_ROL
		YEAR
		BLKSHVAL
		EXDATE
		EXPRIC
		GRNTNUM
		NUMSECUR
		VALUE;
run;
proc download data = remote_grant_data_pre2006 out = pilot.grant_data_pre2006;
run;
/*****************************************************************************/
data remote_outstandingawards;
	set  excomp.outstandingawards;
	cnum = substr(cusip,1,6);
	keep GVKEY
		cnum
		CO_PER_ROL
		YEAR
		EXDATE
		EXPRIC
		OPTS_UNEX_EXER
		OPTS_UNEX_UNEARN
		OPTS_UNEX_UNEXER
		PRCCF
		SHRS_UNVEST_NUM
		SHRS_UNVEST_VAL
		EIP_SHRS_UNVEST_NUM
		EIP_SHRS_UNVEST_VAL;
run;
proc sort data=remote_outstandingawards;
	by gvkey co_per_rol year;
run;
proc download data = remote_outstandingawards out = pilot.outstandingawards;
run;
/*****************************************************************************/
endrsubmit;
signoff;
/*************************
proc sort data=pilot.bs; 			
run;
data pilot.div;
	set pilot.bs; 					*This dataset is created and downloaded above
	by gvkey fyear;
	d_yld = dvpsx_f / prcc_f;
	lag_yr = lag(fyear);
	lag_gv = lag(gvkey);
	if (gvkey=lag_gv) & (fyear=lag_yr) then delete;
	if prcc_f = '.' then delete;
	S = prcc_f;
	drop lag_yr lag_gv prcc_f dvpsx_f;
run;
/*************************
data pilot.daily_sd;
	set general.daily_sd; 			*This dataset is created by SAS script "Daily stock return data"
	if _freq_ < 100 then delete;
	bs_sigma = ln_ret_StdDev * (sqrt(252));
	fyear = year;
	keep cnum fyear bs_sigma;
run;
/*************************/
/* This portion of the script calculates delta and vega for fiscal years 2006 to present*/
/* Full information is available for these years on option portfolios */
data outstanding;
	set pilot.outstandingawards;
	ex_year = year(exdate);
	T = ex_year - year;
	fyear = year;
	if OPTS_UNEX_EXER = '.' then OPTS_UNEX_EXER = 0;
	if OPTS_UNEX_UNEXER = '.' then OPTS_UNEX_UNEXER = 0;
	if SHRS_UNVEST_VAL = '.' then SHRS_UNVEST_VAL = 0;
	total_options =  OPTS_UNEX_EXER + OPTS_UNEX_UNEXER;
	keep co_per_rol fyear gvkey cnum total_options T EXPRIC SHRS_UNVEST_VAL PRCCF;
run;
proc sort data=pilot.daily_sd;
	by cnum fyear;
run;
proc sort data=outstanding;
	by cnum fyear;
run;
data merge_a;
	merge outstanding(in=x) 	pilot.daily_sd(in=y);
	by cnum fyear;
	if x=1;
run;
proc sort data=outstanding;
	by gvkey co_per_rol fyear;
run;
proc sort data=pilot.div;
	by gvkey fyear;
run;
proc sort data=merge_a;
	by gvkey fyear;
run;
data merge_b;
	merge merge_a(in=x) pilot.div(in=y);
	by gvkey fyear;
	if x=1;
run;
data rf;
	set pilot.riskfree;
	fyear = year;
	drop year;
run;
proc sort data=rf;
	by fyear;
run;
proc sort data=merge_b;
	by fyear;
run;
data merge_c;
	merge merge_b(in=x) rf(in=y);
	by fyear;
	if x=1;
run;
proc sort data = merge_c;
	by T;
run;
data merge_d;
	set merge_c;
	if T = '.' then delete;
	if T < 0 then delete;
	if T < 1 then T = 0.5;
	if T < 2 then rf = RF1;
	if T = 2 then rf = RF2;
	if T = 3 then rf = RF3;
	if T > 3 then rf = RF5;
	if T > 6 then rf = RF7;
	if T > 8 then rf = RF10;
	if bs_sigma = '.' then delete;
	drop RF1 RF2 RF3 RF5 RF7 RF10;
run;
proc sort data = merge_d;
	by cnum co_per_rol fyear;
run;
data vega_calc_outstanding;
	set merge_d;
	Z = ((log(S/EXPRIC)) + (T*(rf-d_yld+((bs_sigma*bs_sigma)/2))))
		/(bs_sigma * sqrt(T));
	N = cdf('normal',Z,0,1);
	N_prime = pdf('normal',Z,0,1);
	delta = exp(-d_yld*T)*N*(S/100);
	vega = exp(-d_yld*T)*N_prime*S*sqrt(T)*.01;
	contract_delta = delta*total_options;
	contract_vega = vega*total_options;
run;
proc sort data = vega_calc_outstanding;
	by gvkey co_per_rol fyear;
run;
proc means noprint data = vega_calc_outstanding;
	var total_options contract_vega contract_delta SHRS_UNVEST_VAL;
	by gvkey co_per_rol fyear;
	output out=outstanding_totals sum= ;
run;
data shares_held;
	set pilot.anncomp;
	fyear = year;
	keep gvkey co_per_rol fyear shrown_excl_opts prcc_f;
run;
proc sort data =  shares_held;
	by gvkey co_per_rol fyear;
run;
proc sort data =  outstanding_totals;
	by gvkey co_per_rol fyear;
run;
data pilot.outstanding_totals;
	merge outstanding_totals(in=x) shares_held(in=y);
	by gvkey co_per_rol fyear;
	if x=1;
	if 	shrown_excl_opts = '.' then shrown_excl_opts = 0;
	total_delta = contract_delta + 	(shrown_excl_opts*prcc_f*.01) + (SHRS_UNVEST_VAL*.01);
	total_vega = contract_vega;
	keep gvkey co_per_rol fyear total_delta total_vega;
run;

/*************************/
/* This portion of the script calculates delta and vega for fiscal years prior to 2006*/
/* Full information is not available for these years so delta and vega are calculated */
/* using the OA method from Core and Guay (2002) */
data new_grants;
	set pilot.grant_data_pre2006;
	ex_year = year(exdate);
	T = ex_year - year;
	fyear = year;
	keep co_per_rol grntnum numsecur fyear gvkey cnum T EXPRIC;
run;
/*************************/
proc sort data=pilot.daily_sd;
	by cnum fyear;
run;
proc sort data=new_grants;
	by cnum fyear;
run;
data vega_calc_a;
	merge new_grants(in=x) 	pilot.daily_sd(in=y);
	by cnum fyear;
	if x=1;
run;
proc sort data=pilot.div;
	by gvkey fyear;
run;
proc sort data=vega_calc_a;
	by gvkey fyear;
run;
data vega_calc_b;
	merge vega_calc_a(in=x) pilot.div(in=y);
	by gvkey fyear;
	if x=1;
run;
data rf;
	set pilot.riskfree;
	fyear = year;
	drop year;
run;
proc sort data=rf;
	by fyear;
run;
proc sort data=vega_calc_b;
	by fyear;
run;
data vega_calc_c;
	merge vega_calc_b(in=x) rf(in=y);
	by fyear;
	if x=1;
run;
proc sort data = vega_calc_c;
	by T;
run;
data vega_calc_d;
	set vega_calc_c;
	if T = '.' then delete;
	if T < 0 then delete;
	if T < 1 then T = 0.5;
	if T < 2 then rf = RF1;
	if T = 2 then rf = RF2;
	if T = 3 then rf = RF3;
	if T > 3 then rf = RF5;
	if T > 6 then rf = RF7;
	if T > 8 then rf = RF10;
	if bs_sigma = '.' then delete;
	drop RF1 RF2 RF3 RF5 RF7 RF10;
run;
data vega_calc_e;
	set vega_calc_d;
	Z = ((log(S/EXPRIC)) + (T*(rf-d_yld+((bs_sigma*bs_sigma)/2))))
		/(bs_sigma * sqrt(T));
	N = cdf('normal',Z,0,1);
	N_prime = pdf('normal',Z,0,1);
	delta = exp(-d_yld*T)*N*(S/100);
	vega = exp(-d_yld*T)*N_prime*S*sqrt(T)*.01;
	contract_delta = delta*numsecur;
	contract_vega = vega*numsecur;
	int_val = (S-EXPRIC)*numsecur;
	if int_val < 0 then int_val = 0;
run;
proc sort data = vega_calc_e;
	by co_per_rol gvkey fyear;
run;
proc means noprint data = vega_calc_e;
	var numsecur contract_vega contract_delta int_val;
	by co_per_rol gvkey fyear;
	output out=new_opts_totals sum= ;
run;
/*************************/
data prev_grants;
	set pilot.anncomp;
	fyear = year;
	if shrown_excl_opts = '.' then shrown_excl_opts = 0;
	if STOCK_UNVEST_VAL = '.' then STOCK_UNVEST_VAL = 0;
	if EIP_UNEARN_VAL = '.' then EIP_UNEARN_VAL = 0;
	if opt_unex_exer_est_val < 0 then opt_unex_exer_est_val = 0;
	if opt_unex_unexer_est_val 	< 0 then opt_unex_unexer_est_val = 0;
	if opt_unex_exer_est_val = '.' then opt_unex_exer_est_val = 0;
	if opt_unex_unexer_est_val = '.' then opt_unex_unexer_est_val = 0;
	total_option_value =  opt_unex_exer_est_val +  opt_unex_unexer_est_val;
	total_share_value = (shrown_excl_opts*prcc_f) + stock_unvest_val;
	total_performance_value =  eip_unearn_val;
	keep co_per_rol execid shrown_excl_opts option_awards_num 
		opt_unex_exer_num opt_unex_unexer_num opt_unex_exer_est_val 
		opt_unex_unexer_est_val gvkey fyear cnum cash_comp total_comp 
		perc_cash_comp salary total_option_value total_share_value total_performance_value;
run;
proc sort data = prev_grants;
	by co_per_rol gvkey fyear;
run;
proc sort data = new_opts_totals;
	by co_per_rol gvkey fyear;
run;
data all_grants;
	merge prev_grants new_opts_totals;
	by co_per_rol gvkey fyear;
	if numsecur = '.' then numsecur = 0;
	if int_val = '.' then int_val = 0;
	if contract_delta = '.' then contract_delta = 0;
	if contract_vega = '.' then contract_vega = 0;
	unex_num_excl_new =  max(opt_unex_unexer_num - numsecur,0);
	unex_val_excl_new =  max(opt_unex_unexer_est_val - int_val,0);
	if unex_num_excl_new = 0 then unex_val_excl_new = 0;
	drop _type_ _freq_;
run;
proc sort data=pilot.div;
	by gvkey fyear;
run;
proc sort data=all_grants;
	by gvkey fyear;
run;
data vega_calc_f;
	merge all_grants(in=x) pilot.div(keep=gvkey fyear S d_yld);
	by gvkey fyear;
	if x=1;
run;
proc sort data=rf;
	by fyear;
run;
proc sort data=vega_calc_f;
	by fyear;
run;
data vega_calc_g;
	merge vega_calc_f(in=x) rf(keep=fyear RF7 RF10);
	by fyear;
	if x=1;
run;
proc sort data=pilot.daily_sd;
	by cnum fyear;
run;
proc sort data=vega_calc_g;
	by cnum fyear;
run;
data vega_calc_h;
	merge vega_calc_g(in=x) pilot.daily_sd(in=y);
	by cnum fyear;
	if x=1;
run;
data pilot.vega_oa;
	set vega_calc_h;
	unexer_est_strike =  S - (unex_val_excl_new/unex_num_excl_new);
	exer_est_strike = S - (opt_unex_exer_est_val/opt_unex_exer_num);

	unexer_Z = 	((log(S/unexer_est_strike)) 
				+ (9*(RF10-d_yld+((bs_sigma*bs_sigma)/2))))
				/(bs_sigma * sqrt(9));
	unexer_N = cdf('normal',unexer_Z,0,1);
	unexer_N_prime = pdf('normal',unexer_Z,0,1);
	unexer_delta = exp(-d_yld*9)*unexer_N*(S/100);
	unexer_vega = exp(-d_yld*9)*unexer_N_prime*S*sqrt(9)*.01;
	unexer_total_delta = unexer_delta*unex_num_excl_new;
	unexer_total_vega = unexer_vega*unex_num_excl_new;
	if unexer_total_delta = '.' then unexer_total_delta = 0;
	if unexer_total_vega = '.' then unexer_total_vega = 0;

	exer_Z = 	((log(S/exer_est_strike)) 
				+ (6*(RF7-d_yld+((bs_sigma*bs_sigma)/2))))
				/(bs_sigma * sqrt(6));
	exer_N = cdf('normal',exer_Z,0,1);
	exer_N_prime = pdf('normal',exer_Z,0,1);
	exer_delta = exp(-d_yld*6)*exer_N*(S/100);
	exer_vega = exp(-d_yld*6)*exer_N_prime*S*sqrt(6)*.01;
	exer_total_delta = exer_delta*opt_unex_exer_num;
	exer_total_vega = exer_vega*opt_unex_exer_num;
	if exer_total_delta = '.' then exer_total_delta = 0;
	if exer_total_vega = '.' then exer_total_vega = 0;

	total_delta_oa = (contract_delta + unexer_total_delta 
				+ exer_total_delta) + (shrown_excl_opts*S*0.01);
	total_vega_oa = contract_vega + unexer_total_vega + exer_total_vega;

	if total_vega_oa < 0 then delete;
	if total_delta_oa < 0 then delete;

run;
proc sort data=pilot.vega_oa;
	by gvkey co_per_rol fyear;
run;
proc sort data=pilot.outstanding_totals;
	by gvkey co_per_rol fyear;
run;


